package io.data2viz.charts.config

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.mark.Mark
import io.data2viz.charts.core.*
import io.data2viz.viz.HasMetaKeys
import io.data2viz.viz.KMouseEvent


/**
 * [Chart] events configuration.
 *
 * - zoom
 * - pan
 * - highlight
 * - selection
 */
public interface EventConfig {

    /**
     * Define the behavior for highlighting a [Mark].
     */
    public var highlightMode: HighlightMode

    /**
     * Define how the [Mark] sign will "trigger" (rollover, nearest, ...)
     */
    public var triggerMode: TriggerMode

    /**
     * Define the behavior for selecting a [Mark].
     */
    public var selectionMode: SelectionMode

    //todo should be in zoom config?
    /**
     * Define the zoom behavior.
     */
    public var zoomMode: ZoomMode

    /**
     * Allow for zooming with the mouse wheel (when available) given a [HasMetaKeys] event.
     *
     * Examples:
     * - To disable mouse wheel zoom completely just set the variable to { false }.
     * - To enable mouse wheel zoom without meta-key shortcut just set the variable to { true }.
     * - To enable mouse wheel zoom for both ALT or SHIFT keys set this to { altKey || shiftKey }.
     *
     * Defaults to { ctrlKey }.
     */
    public var zoomEnableOnMouseWheel: HasMetaKeys.() -> Boolean

    /**
     * Define the panning behavior.
     */
    public var panMode: PanMode


    /**
     * Defines the [UserAction] started on a mouse down.
     *
     * Default implementation is :
     *
     * ```
     * { event ->
     *    when {
     *        event.ctrlKey || event.metaKey   -> UserAction.Zoom
     *        event.altKey                     -> UserAction.Pan
     *        else                             -> UserAction.Select
     *    }
     * }
     * ```
     */
    public var getUserActionOnMouseDown: (KMouseEvent) -> UserAction?

//    /**
//     * Defines the [UserAction] started on a mouse wheel.
//     *
//     * Default implementation is :
//     *
//     * ```
//     * { event ->
//     *    when {
//     *        event.ctrlKey || event.metaKey   -> UserAction.Zoom
//     *        event.altKey                     -> UserAction.Pan
//     *        else                             -> UserAction.Select
//     *    }
//     * }
//     * ```
//     */
//    public var getUserActionOnMouseWheel: (KMouseEvent) -> UserAction?
}

