package io.data2viz.charts.event

import io.data2viz.math.Percent

/**
 * A "zoom event" is triggered when a user zoom or dezoom on a [Chart].
 * A zoom event has a zoomOrigin which is relative to the charting zone, and a zoomFactor (dezooming < 100% < zooming).
 *
 * @param [zoomOriginX] the relative zoom origin in [Percent] along the X-axis.
 * @param [zoomOriginY] the relative zoom origin in [Percent] along the Y-axis.
 * @param [zoomFactorX] the zoom ratio in [Percent] on the X dimension.
 * @param [zoomFactorY] the zoom ratio in [Percent] on the Y dimension.
 */
public class ZoomEvent internal constructor(
    public val zoomOriginX: Percent,
    public val zoomOriginY: Percent,
    public val zoomFactorX: Percent,
    public val zoomFactorY: Percent
) : ChartEvent {

    override var preventDefault: Boolean = false

    override fun toString(): String = "Zooming from (${zoomOriginX} - ${zoomOriginY}), Zooming in (${zoomFactorX} - ${zoomFactorY})"
}
